#include <PalmOS.h>
#include "ATTOK.h"

typedef struct 
	{
	UInt16 On;
	UInt16 Off;
	UInt16 AutoEnbleF;
	UInt16 isTTF;
	UInt16 withJOSF;
	} ATTOKPreferenceType;

#define appFileCreator			'ATTK'
#define appVersionNum			0x02
#define appPrefID				0x01
#define appPrefVersionNum		0x01

#define isTT					0x01	//Ftr
#define withJOS					0x02	//Ftr

static void SetATTOKPreference(ATTOKPreferenceType *prefs)
{
	PrefSetAppPreferences (appFileCreator, appPrefID, appPrefVersionNum, 
		prefs, sizeof (ATTOKPreferenceType), false);
}

static void GetATTOKPreference(ATTOKPreferenceType *prefs)
{
	UInt16 prefsSize;

	prefsSize = sizeof(ATTOKPreferenceType);
	if (PrefGetAppPreferences(appFileCreator, appPrefID, prefs, &prefsSize, false) == 
		noPreferenceFound)
		{
			prefs->On         = 0;
			prefs->Off        = 1;
			prefs->AutoEnbleF = 0;
			prefs->isTTF      = 0;
			prefs->withJOSF   = 1;

			SetATTOKPreference(prefs);
		}
}

static void Disable(void)
{
	UInt16 cardNo;
	LocalID dbID;
	SysCurAppDatabase (&cardNo, &dbID);
	SysNotifyUnregister(cardNo, dbID, sysNotifyEventDequeuedEvent, sysNotifyNormalPriority);
}

static void Enable(void)
{
	UInt16 cardNo;
	LocalID dbID;
	SysCurAppDatabase (&cardNo, &dbID);
	SysNotifyRegister (cardNo, dbID, sysNotifyEventDequeuedEvent, NULL, sysNotifyNormalPriority, NULL);

	ATTOKPreferenceType prefs;
	GetATTOKPreference(&prefs);
	FtrSet (appFileCreator, isTT   , prefs.isTTF   );
	FtrSet (appFileCreator, withJOS, prefs.withJOSF);
}

static void GetCtrlValue(FormPtr frmP)
{
	ATTOKPreferenceType prefs;

	prefs.On         = FrmGetControlValue (frmP, FrmGetObjectIndex (frmP, MainONPushButton     ));
																		// !0:ON 0:OFF
	prefs.Off        = FrmGetControlValue (frmP, FrmGetObjectIndex (frmP, MainOFFPushButton    ));
																		// !0:ON 0:OFF
	prefs.AutoEnbleF = FrmGetControlValue (frmP, FrmGetObjectIndex (frmP, MainEnblRstCheckbox  ));
																		// !0:ON 0:OFF
	prefs.isTTF      = FrmGetControlValue (frmP, FrmGetObjectIndex (frmP, MainTTCheckbox       ));
																		// !0:ON 0:OFF
	prefs.withJOSF   = FrmGetControlValue (frmP, FrmGetObjectIndex (frmP, MainJOSCheckbox      ));
																		// !0:ON 0:OFF
	if ((prefs.isTTF == 0) && (prefs.withJOSF == 0))
	{
		prefs.On = 0;
		prefs.Off = 1;
	}

	SetATTOKPreference(&prefs);
	
	if (prefs.On != 0)
	{
		Enable();
	}
	else
	{
		Disable();
	}
}

static void SetCtrlValue(FormPtr frmP)
{
	ATTOKPreferenceType prefs;
	GetATTOKPreference(&prefs);

	FrmSetControlValue (frmP, FrmGetObjectIndex (frmP, MainONPushButton     ), prefs.On        );
	FrmSetControlValue (frmP, FrmGetObjectIndex (frmP, MainOFFPushButton    ), prefs.Off       );
	FrmSetControlValue (frmP, FrmGetObjectIndex (frmP, MainEnblRstCheckbox  ), prefs.AutoEnbleF);
	FrmSetControlValue (frmP, FrmGetObjectIndex (frmP, MainTTCheckbox       ), prefs.isTTF     );
	FrmSetControlValue (frmP, FrmGetObjectIndex (frmP, MainJOSCheckbox      ), prefs.withJOSF  );
}

static UInt32 StarterPalmMain(UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags)
{
//	Err error;

	switch (cmd)
		{
		case sysAppLaunchCmdNotify:
			SysNotifyParamType* NtfyPrm = (SysNotifyParamType*)cmdPBP;
			switch (NtfyPrm->notifyType)
				{
				case sysNotifyEventDequeuedEvent:
					EventPtr DequedEvent;
					DequedEvent = (EventType*)NtfyPrm->notifyDetailsP;
					if (DequedEvent->eType == 0x2361)		//call suisokuhenkan vchr 0x6123
					{
						UInt32 isTTF;
						FtrGet (appFileCreator, isTT, &isTTF);
						if (isTTF != 0)
						{
							FormType *frmP = FrmGetActiveForm ();
							if (frmP != NULL)
							{
								UInt16 objIndex = FrmGetFocus (frmP);
								if (objIndex != noFocus)
								{
									if(FrmGetObjectType (frmP, objIndex) == frmTableObj)
									{
										DequedEvent->eType = nilEvent;
									}
								}
							}
						}
					}

					if ( DequedEvent->eType == 0x0400 )
					{
						if ( (DequedEvent->data.keyDown.chr == 0x1001) || (DequedEvent->data.keyDown.chr == 0x1101) )//vchrKeyboardAlpha 0x0110 vchrKeyboardNumeric 0x0111
						{
							UInt32 withJOSF;
							FtrGet (appFileCreator, withJOS, &withJOSF);
							if (withJOSF != 0)
							{
								FormPtr frmP = FrmGetActiveForm ();
								UInt16 Focused = FrmGetFocus (frmP);
								if (Focused == noFocus)
								{
									DequedEvent->eType = nilEvent;
								}
								else
								{
									FormObjectKind Type = FrmGetObjectType (frmP, Focused);
									if ( (Type != frmFieldObj) && (Type != frmTableObj) )
									{
										DequedEvent->eType = nilEvent;
										break;
									}
									if (Type == frmTableObj)
									{
										if (TblGetCurrentField ((TableType*)FrmGetObjectPtr (frmP, Focused)) == NULL)
										DequedEvent->eType = nilEvent;
									}
								}
							}
						}
					}
					break;

				}
			break;

		case sysAppLaunchCmdNormalLaunch:
			FormPtr frmP = FrmInitForm (MainForm);		//Initialize dialogue form
			SetCtrlValue(frmP);				 			//Set control value
			if (FrmDoDialog (frmP) == MainDoneButton)	//Draw dialogue form
				GetCtrlValue(frmP);						//Get control value
			break;

		case sysAppLaunchCmdSystemReset:
			ATTOKPreferenceType prefs;					//Check Auto Enable
			GetATTOKPreference(&prefs);
			if (prefs.On != 0)
			{
				if (prefs.AutoEnbleF != 0)
					Enable();						//activate
				else
				{
					prefs.On  = 0;
					prefs.Off = 1;
					SetATTOKPreference(&prefs);
				}
			}
			break;

		default:
			break;

		}
	
	return errNone;
}

UInt32 PilotMain( UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags)
{
	return StarterPalmMain(cmd, cmdPBP, launchFlags);
}
